@@ -1,6 +1,6 @@
 /* Common target dependent code for GDB on Alpha systems.
-   Copyright (C) 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2002, 2003, 2007,
-   2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1993-1996, 1998-2000, 2002-2003, 2007-2012 Free
+   Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -58,9 +58,9 @@ struct regcache;
    
    The virtual argument pointer is pointing to the bottom of the argument
    transfer area, which is located immediately below the virtual frame
-   pointer. Its size is fixed for the native compiler, it is either zero
+   pointer.  Its size is fixed for the native compiler, it is either zero
    (for the no arguments case) or large enough to hold all argument registers.
-   gcc uses a variable sized argument transfer area. As it has
+   gcc uses a variable sized argument transfer area.  As it has
    to stay compatible with the native debugging tools it has to use the same
    virtual argument pointer and adjust the argument offsets accordingly.
    
@@ -86,7 +86,8 @@ struct gdbarch_tdep
   /* NOTE: cagney/2004-04-30: Do not copy/clone this code.  Instead
      look at tramp-frame.h and other simplier per-architecture
      sigtramp unwinders.  */
-  int (*pc_in_sigtramp) (struct gdbarch *gdbarch, CORE_ADDR pc, char *name);
+  int (*pc_in_sigtramp) (struct gdbarch *gdbarch, CORE_ADDR pc,
+			 const char *name);
 
   /* If TYPE will be returned in memory, return true.  */
   int (*return_in_memory) (struct type *type);
@@ -99,7 +100,7 @@ struct gdbarch_tdep
   int jb_pc;			/* Offset to PC value in jump buffer.
 				   If htis is negative, longjmp support
 				   will be disabled.  */
-  size_t jb_elt_size;		/* And the size of each entry in the buf. */
+  size_t jb_elt_size;		/* And the size of each entry in the buf.  */
 };
 
 extern unsigned int alpha_read_insn (struct gdbarch *gdbarch, CORE_ADDR pc);
